--[[
Changelog:
3.16 (release)
- Fix errors in scenarios.
3.15 (release)
- Fix errors when instance group members leave.
3.14 (release)
- Fix aggro highlight option.
3.13 (internal)
- Fix partyid offset for compactunitframes
3.12 (internal)
- Fix specMap according to global_spec_id instead of specIndex
3.11 (internal)
- Replace LibGroupTalents, LibTalentQuerry functionality with LibGroupInSpecT-1.0
3.10 (internal))
- WoW 5.0.4/MoP compatibility
- LibGroupTalents, LibTalentQuerry functionality commented out until/if the libs get updated for Mists of Pandaria
- Improve role detection by talents for the player
3.06
- More Cataclysm compatibility
- Manually setting roles.
3.05
- Cataclysm beta compatibility
3.04(internal)
- move to flash frame replacement functions to avoid tainting (Jerry's code with small addition)
3.03(internal)
- split event processing
- create a priority system for role overwrites
3.02(internal)
- add /fragile mode for showing the mode select popup
- add ACTIVE_TALENT_GROUP_CHANGED monitoring
]]
-- GetCVarBool("useCompactPartyFrames")
-- frame.aggroHighlight -- for compact
-- frame.threatIndicator -- for classic

local Fragile3 = LibStub("AceAddon-3.0"):GetAddon("Fragile3",true)
if not Fragile3 then
	Fragile3 = LibStub("AceAddon-3.0"):NewAddon("Fragile3", "AceEvent-3.0", "AceConsole-3.0", "LibSink-2.0")
end
local L = LibStub("AceLocale-3.0"):GetLocale("Fragile3", false)
local LSM = LibStub("LibSharedMedia-3.0")
local LGI = LibStub("LibGroupInSpecT-1.0")
local LBZ = LibStub("LibBanzai-2.0")
Fragile3.version = GetAddOnMetadata("Fragile3", "Version").."."..GetAddOnMetadata("Fragile3", "X-MinorVersion")
_G["Fragile3"] = Fragile3

-- Localize keybind descriptions
BINDING_HEADER_FRAGILE3 = "Fragile3"
BINDING_NAME_SETTANK = L["Set Tank"]
BINDING_NAME_SETHEALER = L["Set Healer"]
BINDING_NAME_SETDPS = L["Set Damager"]
BINDING_NAME_CLEARROLE = L["Clear Role"]

-- Add our sounds to sharedmedia
LSM:Register("sound", "FL3: "..L["(Aggro) Unknown"], [[Interface\AddOns\Fragile3\Sounds\aggro_unknown.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Unknown Male"], [[Interface\AddOns\Fragile3\Sounds\aggro_male.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Unknown Female"], [[Interface\AddOns\Fragile3\Sounds\aggro_fem.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Healer Male"], [[Interface\AddOns\Fragile3\Sounds\healeraggro_male.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Healer Female"], [[Interface\AddOns\Fragile3\Sounds\healeraggro_fem.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Damage Male"], [[Interface\AddOns\Fragile3\Sounds\dpsaggro_male.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Damage Female"], [[Interface\AddOns\Fragile3\Sounds\dpsaggro_fem.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Self Male"], [[Interface\AddOns\Fragile3\Sounds\overaggro_male.mp3]])
LSM:Register("sound", "FL3: "..L["(Aggro) Self Female"], [[Interface\AddOns\Fragile3\Sounds\overaggro_fem.mp3]])

-- locals
local db
local Fragile3Defaults = {
	profile = {
		Enabled				= true,
		UseInRaid			= true,
		Sensitivity		= 1,
		Alerts				= {
			Sound       	= true,
			RoleVoices 		= true,
			GenderVoices	= true,
			Sounds				= {
				aggroUnknown 			= "FL3: "..L["(Aggro) Unknown"],
				aggroUnknownMale 	= "FL3: "..L["(Aggro) Unknown Male"],
				aggroUnknownFem 	= "FL3: "..L["(Aggro) Unknown Female"],
				aggroHealerMale 	= "FL3: "..L["(Aggro) Healer Male"],
				aggroHealerFem 		= "FL3: "..L["(Aggro) Healer Female"],
				aggroDPSMale 			= "FL3: "..L["(Aggro) Damage Male"],
				aggroDPSFem 			= "FL3: "..L["(Aggro) Damage Female"],
				aggroOverMale 		= "FL3: "..L["(Aggro) Self Male"],
				aggroOverFem 			= "FL3: "..L["(Aggro) Self Female"],				
			},
			Message				= true,
			CClassColors  = true,
			SinkOptions 	= {},
			PartyFlash    = true,
			Throttle			= 2,
		},
		TankMode			= {
			HealerAlert		= true,
			DPSAlert			= true,
			UnknownAlert	= true,
		},
		HealerMode		= {
			SelfAlert			= true,
			HealerAlert		= true,
			DPSAlert			= true,
			UnknownAlert	= true,
		},
		DPSMode				= {
			SelfAlert			= true,
			HealerAlert		= true,
			DPSAlert			= false,
			UnknownAlert	= false,
		}
	},
}

local debug = false
local lastAggroAlert 	-- last alert time 
local lastMessage 		-- chatmessage time (debug + addon output)
local lastRoster			-- last roster update time
local groupStatus			-- PARTY, RAID, SOLO
local mode						-- TANK, HEALER, DAMAGER
local roleList = {} 	-- key = guid, value = {ROLE, setter} TANK, HEALER, DAMAGER, UNKNOWN|event
local partyid = {}		-- precache unitids for loops
local raidid = {}			-- precache unitids for loops
local warnstatus = 1  -- status to warn depending on sensitivity setting
local banzaiEvents		-- flag for callback registration/unregistration
local unknownclasscolor = "|cffCCCCCC"
local msgPrefix = {L["Aggro on >>"], L["High Threat on >>"], L["Aggro on >>"]}
for i=1,4 do
	partyid[i] = format("party%d",i)
end
for i=1,40 do
	raidid[i] = format("raid%d",i)
end
local specMap = {
	["WARRIOR"] = {[71]=DAMAGER,[72]=DAMAGER,[73]=TANK},
	["DEATHKNIGHT"] = {[250]=TANK,[251]=DAMAGER,[252]=DAMAGER},
	["PALADIN"] = {[65]=HEALER,[66]=TANK,[70]=DAMAGER},
	["MONK"] = {[268]=TANK,[269]=HEALER,[270]=DAMAGER},
	["PRIEST"] = {[256]=HEALER,[257]=HEALER,[258]=DAMAGER},
	["SHAMAN"] = {[262]=DAMAGER,[263]=DAMAGER,[264]=HEALER},
	["DRUID"] = {[102]=DAMAGER,[103]=DAMAGER,[104]=TANK,[105]=HEALER},
}
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local GENDER_UNKNOWN, GENDER_MALE, GENDER_FEMALE = 1, 2, 3
local SETTER_DEFAULT, SETTER_MANUAL, SETTER_TALENTS, SETTER_LFD, SETTER_CLASS, SETTER_RAIDAPI = 0, 1, 2, 3, 4, 5

-- -- -- 
function Fragile3:CreateFrames()
	-- TODO: Create the pick mode frame (model it after the LFD pick role)
	-- TODO: Create the pick role frame (list with checkboxes and 3 buttons for mass assign role)
end

function Fragile3:OnInitialize()
	-- Create savedvariables
	self.db = LibStub("AceDB-3.0"):New("Fragile3DB", Fragile3Defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	db = self.db.profile
	self:SetSinkStorage(db.Alerts.SinkOptions)
	
	-- not needed for sounds but just in case we add more mediatypes.
	LSM.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateUsedMedia") 

	self:CreateFrames()
	self:SetupOptions()
	
	self.OnInitialize = nil
end

function Fragile3:OnEnable()
	self:RegisterEvent("GROUP_ROSTER_UPDATE")
	self:RegisterEvent("GROUP_JOINED","GROUP_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","GROUP_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_ROLES_ASSIGNED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("ROLE_CHANGED_INFORM")
	LGI.RegisterCallback(self,"GroupInSpecT_Update","GroupTalentUpdate") -- guid, unit, info (table)
	self:GROUP_ROSTER_UPDATE()
	self:DebugPrint("Enabled")
end

function Fragile3:OnDisable()
	self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	if banzaiEvents then
		LBZ.UnregisterCallback(self, Fragile3.BanzaiCallback)
		banzaiEvents = false
	end
	self:DebugPrint("Disabled")
end

function Fragile3:ProfileChanged(event, database, newProfileKey)
	db = database.profile;
end

function Fragile3:UpdateUsedMedia(event, mediatype, key)

end

function Fragile3:Alert(name, class, role, gender, id, groupStatus, threatStatus)
 -- Whether we should alert or not will have been determined in the calling function (Role, groupStatus, Throttle)
 -- This just creates the alerts output according to options and parameters passed. 

	if groupStatus == PARTY then
		if db.Alerts.Message then
			classcolor = RAID_CLASS_COLORS[class] and string.format("|cff%02x%02x%02x",RAID_CLASS_COLORS[class].r*255,RAID_CLASS_COLORS[class].g*255,RAID_CLASS_COLORS[class].b*255) or unknownclasscolor
			local binding = id and "TARGETPARTYMEMBER"..id
			local key1, key2 = binding and GetBindingKey(binding);
			key1 = key1 or key2;
			local key = (binding and key1) and string.format("(%s)",key1) or "";
			local msg = string.format("%s%s%s|r<< %s", (threatStatus>=2) and L["Aggro on >>"] or (threatStatus==1) and L["High Threat on >>"], classcolor, name, key)
			self:Pour(msg,1, 0, 0, nil, 20, "OUTLINE", true)
		end
		if db.Alerts.Sound then
			local sound
			if name == UNIT_YOU then
				if db.Alerts.GenderVoices then
					if gender == GENDER_MALE then
						sound = db.Alerts.Sounds.aggroOverMale
					elseif gender == GENDER_FEMALE then
						sound = db.Alerts.Sounds.aggroOverFem
					else
						sound = db.Alerts.Sounds.aggroUnknown
					end
				else
					sound = db.Alerts.Sounds.aggroOverFem
				end
			else
				if role == HEALER then
					if db.Alerts.RoleVoices then
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroHealerMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroHealerFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroHealerFem
						end
					else
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroUnknownMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroUnknownFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					end
				elseif role == DAMAGER then
					if db.Alerts.RoleVoices then
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroDPSMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroDPSFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroDPSFem
						end
					else
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroUnknownMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroUnknownFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					end
				else -- unknown role
					if db.Alerts.GenderVoices then
						if gender == GENDER_MALE then
							sound = db.Alerts.Sounds.aggroUnknownMale
						elseif gender == GENDER_FEMALE then
							sound = db.Alerts.Sounds.aggroUnknownFem
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					else
						sound = db.Alerts.Sounds.aggroUnknown
					end
				end
			end
			if sound then PlaySoundFile(LSM:Fetch("sound",sound)) end
		end
		if db.Alerts.PartyFlash then
			local partyaggroframe
			if GetCVarBool("useCompactPartyFrames") then
				partyaggroframe = id and (_G["CompactPartyFrameMember"..(id+1)] or _G["CompactRaidFrame"..(id+1)]).aggroHighlight
			else
				partyaggroframe = id and _G["PartyMemberFrame"..id].threatIndicator
			end
			if partyaggroframe then
				partyaggroframe:SetVertexColor(GetThreatStatusColor(threatStatus))
				JerryFrameFlash(partyaggroframe,0.35,0.35,-1)
			end
		end
	elseif groupStatus == RAID then
		if db.Alerts.Message then
			classcolor = RAID_CLASS_COLORS[class] and string.format("|cff%02x%02x%02x",RAID_CLASS_COLORS[class].r*255,RAID_CLASS_COLORS[class].g*255,RAID_CLASS_COLORS[class].b*255) or unknownclasscolor
			local msg = string.format("%s%s%s|r<<",(threatStatus>=2) and L["Aggro on >>"] or (threatStatus==1) and L["High Threat on >>"],classcolor,name)
			self:Pour(msg,1, 0, 0, nil, 20, "OUTLINE", true)
		end
		if db.Alerts.Sound then
			local sound
			if name == UNIT_YOU then
				if db.Alerts.GenderVoices then
					if gender == GENDER_MALE then
						sound = db.Alerts.Sounds.aggroOverMale
					elseif gender == GENDER_FEMALE then
						sound = db.Alerts.Sounds.aggroOverFem
					else
						sound = db.Alerts.Sounds.aggroUnknown
					end
				else
					sound = db.Alerts.Sounds.aggroOverFem
				end
			else
				if role == HEALER then
					if db.Alerts.RoleVoices then
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroHealerMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroHealerFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroHealerFem
						end
					else
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroUnknownMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroUnknownFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					end
				elseif role == DAMAGER then
					if db.Alerts.RoleVoices then
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroDPSMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroDPSFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroDPSFem
						end
					else
						if db.Alerts.GenderVoices then
							if gender == GENDER_MALE then
								sound = db.Alerts.Sounds.aggroUnknownMale
							elseif gender == GENDER_FEMALE then
								sound = db.Alerts.Sounds.aggroUnknownFem
							else
								sound = db.Alerts.Sounds.aggroUnknown
							end
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					end
				else -- unknown role
					if db.Alerts.GenderVoices then
						if gender == GENDER_MALE then
							sound = db.Alerts.Sounds.aggroUnknownMale
						elseif gender == GENDER_FEMALE then
							sound = db.Alerts.Sounds.aggroUnknownFem
						else
							sound = db.Alerts.Sounds.aggroUnknown
						end
					else
						sound = db.Alerts.Sounds.aggroUnknown
					end
				end
			end
			if sound then PlaySoundFile(LSM:Fetch("sound",sound)) end
		end			
	end
end

function Fragile3:GROUP_ROSTER_UPDATE()
	if not lastRoster or (GetTime()-lastRoster) >= TOOLTIP_UPDATE_TIME then
		local noRole
		if IsInRaid() then
			for i=1, GetNumGroupMembers() do
				local _, _, _, _, _, eclass, _, _, _, role = GetRaidRosterInfo(i)
				local unitguid = UnitGUID(raidid[i])
				local isPlayer = UnitIsUnit("player", raidid[i])
				noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
				if role == "MAINTANK" and (noRole or roleList[unitguid][2] <= SETTER_RAIDAPI) then
					roleList[unitguid] = {TANK, SETTER_RAIDAPI}
					if isPlayer then
						mode = roleList[unitguid][1]
					end
				end
				if (eclass == "ROGUE" or eclass == "HUNTER" or eclass == "MAGE" or eclass == "WARLOCK") and (noRole or roleList[unitguid][2] <= SETTER_CLASS) then
					roleList[unitguid] = {DAMAGER, SETTER_CLASS}
					if isPlayer then
						mode = roleList[unitguid][1]
					end
				end
				if noRole or roleList[unitguid][2] <= SETTER_TALENTS then
					-- check specMap table
					if specMap[eclass] then
						if isPlayer then
							local spec = GetSpecialization()
							if spec and spec > 0 then
								roleList[unitguid] = {(GetSpecializationRole(spec)), SETTER_TALENTS}
								mode = roleList[unitguid][1]
							end
						else
							local info = LGI:GetCachedInfo(unitguid)
							local spec = info and info.spec_role
							roleList[unitguid] = spec and {spec, SETTER_TALENTS}
						end
					end
				end 
				if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
			end
			groupStatus = RAID
		elseif IsInGroup() then
			local isTank, isHealer, isDamage;
			local role = UnitGroupRolesAssigned("player")
			isTank,isHealer,isDamage = role == "TANK", role == "HEALER", role == "DAMAGER"
			local unitguid = UnitGUID("player")
			local _, eclass = UnitClass("player")
			noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
			if noRole or roleList[unitguid][2] <= SETTER_LFD then
				roleList[unitguid] = isTank and {TANK, SETTER_LFD} or isHealer and {HEALER, SETTER_LFD} or isDamage and {DAMAGER, SETTER_LFD} or {UNKNOWN, SETTER_DEFAULT}
				mode = roleList[unitguid][1]
			end
			if (eclass == "ROGUE" or eclass == "HUNTER" or eclass == "MAGE" or eclass == "WARLOCK") and (noRole or roleList[unitguid][2] <= SETTER_CLASS) then
				roleList[unitguid] = {DAMAGER, SETTER_CLASS}
				mode = roleList[unitguid][1]
			end
			if noRole or roleList[unitguid][2] <= SETTER_TALENTS then
				-- check specMap table
				if specMap[eclass] then
					local spec = GetSpecialization()
					if spec and spec > 0 then
						roleList[unitguid] = {(GetSpecializationRole(spec)), SETTER_TALENTS}
					end
				end
			end
			if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
			mode = roleList[unitguid][1]
			for i=1, GetNumSubgroupMembers() do
				local unitguid = UnitGUID(partyid[i])
				local _, eclass = UnitClass(partyid[i])
				local isTank, isHealer, isDamage;
				local role = UnitGroupRolesAssigned(partyid[i])
				isTank,isHealer,isDamage = role == "TANK", role == "HEALER", role == "DAMAGER"
				noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
				if noRole or roleList[unitguid][2] <= SETTER_LFD then
					roleList[unitguid] = isTank and {TANK, SETTER_LFD} or isHealer and {HEALER, SETTER_LFD} or isDamage and {DAMAGER, SETTER_LFD} or {UNKNOWN, SETTER_DEFAULT}
				end
				if (eclass == "ROGUE" or eclass == "HUNTER" or eclass == "MAGE" or eclass == "WARLOCK") and (noRole or roleList[unitguid][2] <= SETTER_CLASS) then
					roleList[unitguid] = {DAMAGER, SETTER_CLASS}
				end
				if noRole or roleList[unitguid][2] <= SETTER_TALENTS then
					if not isPlayer then
						if specMap[eclass] then
							local info = LGI:GetCachedInfo(unitguid)
							local spec = info and info.spec_role
							roleList[unitguid] = spec and {spec, SETTER_TALENTS}
						end
					end
				end
				local partyaggroframe
				if GetCVarBool("useCompactPartyFrames") then
					partyaggroframe = i and (_G["CompactPartyFrameMember"..(i+1)] or _G["CompactRaidFrame"..(i+1)]).aggroHighlight
				else
					partyaggroframe = i and _G["PartyMemberFrame"..i].threatIndicator
				end
				if partyaggroframe and JerryFrameIsFlashing(partyaggroframe) then
					JerryFrameFlashStop(partyaggroframe)
				end
				if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
			end
			groupStatus = PARTY
		else
			self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
			if banzaiEvents then
				LBZ.UnregisterCallback(self, Fragile3.BanzaiCallback)
				banzaiEvents = false
			end
			groupStatus = SOLO
		end
 		if groupStatus ~= SOLO then
			if warnstatus >= 0 then
				self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
				if banzaiEvents then
					LBZ.UnregisterCallback(self, Fragile3.BanzaiCallback)
					banzaiEvents = false
				end
			else
				self:UnregisterEvent("UNIT_THREAT_SITUATION_UPDATE")
				if not banzaiEvents then
					LBZ.RegisterCallback(self, Fragile3.BanzaiCallback)
					banzaiEvents = true
				end
			end
			if not mode or mode == UNKNOWN then
				if not Fragile3ModePopup:IsShown() then
					Fragile3ModePopup:Show();
					Fragile3:DebugPrint("Could not determine your role.")
				end
			end
 		end
		lastRoster = GetTime()
	end
end

function Fragile3:PLAYER_ROLES_ASSIGNED()
	local unitguid = UnitGUID("player")
	if not unitguid then return end
	local isTank, isHealer, isDamage;
	local role = UnitGroupRolesAssigned("player")
	isTank,isHealer,isDamage = role == "TANK", role == "HEALER", role == "DAMAGER"
	local noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
	if noRole or roleList[unitguid][2] <= SETTER_LFD then
		roleList[unitguid] = isTank and {TANK, SETTER_LFD} or isHealer and {HEALER, SETTER_LFD} or isDamage and {DAMAGER, SETTER_LFD} or {UNKNOWN, SETTER_DEFAULT}
	end
	mode = roleList[unitguid][1]
end

function Fragile3:ROLE_CHANGED_INFORM(event, ...)
	local whochanged, bywho, oldrole, newrole = ...;
	local unitguid;
	if whochanged == UnitName("player") then
		unitguid = UnitGUID("player")
	end
	if not unitguid then
		for i=1, GetNumSubgroupMembers() do
			if whochanged == UnitName(partyid[i]) then
				unitguid = UnitGUID(partyid[i])
				break;
			end
		end
	end
	if not unitguid then return end
	if newrole == "NONE" then
		roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT}
	else
		local noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
		if noRole or roleList[unitguid][2] <= SETTER_LFD then
			if newrole == "TANK" then
				roleList[unitguid] = {TANK, SETTER_LFD}
			elseif newrole == "HEALER" then
				roleList[unitguid] = {HEALER, SETTER_LFD}
			elseif newrole == "DAMAGER" then
				roleList[unitguid] = {DAMAGER, SETTER_LFD}
			end
		end
	end
	if unitguid == UnitGUID("player") then
		mode = roleList[unitguid][1]
	end
end
function Fragile3:ACTIVE_TALENT_GROUP_CHANGED()
	local unitguid = UnitGUID("player")
	local _, eclass = UnitClass("player")
	local noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
	if noRole or roleList[unitguid][2] <= SETTER_TALENTS then
		-- check specMap table
		if specMap[eclass] then
			local spec = GetSpecialization()
			if spec and spec > 0 then
				roleList[unitguid] = {(GetSpecializationRole(spec)), SETTER_TALENTS}
			end
		end
		if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
		mode = roleList[unitguid][1]
	end
end

function Fragile3:GroupTalentUpdate(unitguid, unit, info)
	local noRole = not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN
	if noRole or roleList[unitguid][2] <= SETTER_TALENTS then
		if info and info.class and specMap[info.class] then
			roleList[unitguid] = info.spec_role and {info.spec_role, SETTER_TALENTS}
			if UnitIsUnit(unit, "player") then
				mode = roleList[unitguid][1]
			end
		end
	end
end

--[[function Fragile3:RoleUpdateComplete(...)	-- guid1 guid2 etc
	local n = select("#", ...)
	if n > 0 then
		local playerguid = UnitGUID("player")
		for i = 1, n do
			local unitguid = (select(i,...))
			if not roleList[unitguid] or roleList[unitguid][1] == UNKNOWN or roleList[unitguid][2] <= SETTER_TALENTS then
 				local tmpRole = LGT:GetGUIDRole(unitguid)
				roleList[unitguid] = (tmpRole == "tank") and {TANK, SETTER_TALENTS} 
					or (tmpRole == "melee" or tmpRole == "caster") and {DAMAGER, SETTER_TALENTS}
					or (tmpRole == "healer") and {HEALER, SETTER_TALENTS}
					or {UNKNOWN, SETTER_DEFAULT}
				if playerguid == unitguid then
					mode = roleList[unitguid][1]
				end
			end
		end
	end
end]]

function Fragile3:UnitAggroChanged(unitId, status)
	if status and status > warnstatus then -- aggro or high threat (depends on sensitivity setting)
		if not lastAggroAlert or (GetTime() - lastAggroAlert) >= db.Alerts.Throttle then
			if groupStatus == PARTY then
				local alert = false
				local name = UnitName(unitId)
				local unitguid = UnitGUID(unitId)
				if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
				if UnitIsUnit("player", unitId) then
					if (mode == HEALER and db.HealerMode.SelfAlert) or (mode == DAMAGER and db.DPSMode.SelfAlert) then
						alert = true
						name = UNIT_YOU
					end
				else
					if mode == TANK and ((roleList[unitguid][1] == HEALER and db.TankMode.HealerAlert)  -- nil index error (pet?)
					or (roleList[unitguid][1] == DAMAGER and db.TankMode.DPSAlert) 
					or (roleList[unitguid][1] == UNKNOWN and db.TankMode.UnknownAlert)) then 
						alert = true
					else
						if mode == HEALER and ((roleList[unitguid][1] == HEALER and db.HealerMode.HealerAlert) 
						or (roleList[unitguid][1] == DAMAGER and db.HealerMode.DPSAlert) 
						or (roleList[unitguid][1] == UNKNOWN and db.HealerMode.UnknownAlert)) then
							alert = true
						elseif mode == DAMAGER and ((roleList[unitguid][1] == HEALER and db.DPSMode.HealerAlert) 
						or (roleList[unitguid][1] == DAMAGER and db.DPSMode.DPSAlert) 
						or (roleList[unitguid][1] == UNKNOWN and db.DPSMode.UnknownAlert)) then
							alert = true
						end
					end
				end
				if alert then
					local _, eclass = UnitClass(unitId)
					local role = roleList[unitguid][1] or UNKNOWN
					local gender = UnitSex(unitId)
					local partyid = string.match(unitId, "party(%d)")
					self:Alert(name, eclass, role, gender, partyid, groupStatus, status)
				end
	
			elseif groupStatus == RAID then
				local alert = false
				local name = UnitName(unitId)
				local unitguid = UnitGUID(unitId)
				if not roleList[unitguid] then roleList[unitguid] = {UNKNOWN, SETTER_DEFAULT} end
				if UnitIsUnit("player", unitId) then
					if (mode == HEALER and db.HealerMode.SelfAlert) or (mode == DAMAGER and db.DPSMode.SelfAlert) then
						alert = true
						name = UNIT_YOU
					end
				else
					if mode == TANK and ((roleList[unitguid][1] == HEALER and db.TankMode.HealerAlert) 
					or (roleList[unitguid][1] == DAMAGER and db.TankMode.DPSAlert) 
					or (roleList[unitguid][1] == UNKNOWN and db.TankMode.UnknownAlert)) then
						alert = true
					else
						if mode == HEALER and ((roleList[unitguid][1] == HEALER and db.HealerMode.HealerAlert) 
						or (roleList[unitguid][1] == DAMAGER and db.HealerMode.DPSAlert) 
						or (roleList[unitguid][1] == UNKNOWN and db.HealerMode.UnknownAlert)) then
							alert = true
						elseif mode == DAMAGER and ((roleList[unitguid][1] == HEALER and db.DPSMode.HealerAlert) 
						or (roleList[unitguid][1] == DAMAGER and db.DPSMode.DPSAlert) 
						or (roleList[unitguid][1] == UNKNOWN and db.DPSMode.UnknownAlert)) then
							alert = true
						end
					end
				end
				if alert then
					local _, eclass = UnitClass(unitId)
					local role = roleList[unitguid][1] or UNKNOWN
					local gender = UnitSex(unitId)
					self:Alert(name, eclass, role, gender, nil, groupStatus, status);
				end
			end
			lastAggroAlert = GetTime()
		end
	elseif not status or status < 1 then -- aggro loss
		if groupStatus == PARTY then
			local partyid = string.match(unitId, "party(%d)")
			local partyaggroframe
			if GetCVarBool("useCompactPartyFrames") then
				partyaggroframe = partyid and (_G["CompactPartyFrameMember"..(partyid+1)] or _G["CompactRaidFrame"..(partyid+1)]).aggroHighlight
			else
				partyaggroframe = partyid and _G["PartyMemberFrame"..partyid].threatIndicator
			end
			if partyaggroframe and JerryFrameIsFlashing(partyaggroframe) then
				JerryFrameFlashStop(partyaggroframe)
			end
			
		elseif groupStatus == RAID then
			
		end
	end
end

function Fragile3.BanzaiCallback(aggro, name, ...)
	if groupStatus == RAID and not db.UseInRaid then return end
	local unitId = ...;
	local status = (aggro==1) and 2 or nil
	Fragile3:UnitAggroChanged(unitId, status)
end

function Fragile3:UNIT_THREAT_SITUATION_UPDATE(event, ...)
	local unitId = ...;
	-- check the trivial cases to bail out early
	if not (unitId) or not UnitExists(unitId) then return end
	if groupStatus == RAID and not db.UseInRaid then return end
	
	local status = UnitThreatSituation(unitId)
	self:UnitAggroChanged(unitId, status)
end

function Fragile3:DumpRoles()
	if groupStatus == SOLO then Fragile3:DebugPrint("Not in group") return end
	if groupStatus == PARTY then
		local playerguid = UnitGUID("player")
		local playername = UnitName("player")
		local playerrole = roleList[playerguid][1]
		local playersetter = roleList[playerguid][2]
		if playerrole then
			Fragile3:DebugPrint(playername..":"..playerrole..":"..playersetter)
		end
		for i=1, GetNumSubgroupMembers() do
			local unitguid = UnitGUID(partyid[i])
			local name = UnitName(partyid[i])
			local role = roleList[unitguid][1]
			local setter = roleList[unitguid][2]
			if role then
				Fragile3:DebugPrint(name..":"..role..":"..setter)
			end
		end
	end
	if groupStatus == RAID then
		for i=1, GetNumGroupMembers() do
			local unitguid = UnitGUID(raidid[i])
			local name = UnitName(raidid[i])
			local role = roleList[unitguid][1]
			local setter = roleList[unitguid][2]
			if role then
				Fragile3:DebugPrint(name..":"..role..":"..setter)
			end
		end
	end
end

function Fragile3:DebugPrint(msg)
	if DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage("|cff3399ffFragile:|r "..tostring(msg))
	end
end

function Fragile3:SetRole(role)
	if groupStatus == SOLO then 
		self:DebugPrint("Cannot set a role when "..SOLO)
		return 
	end
	if not UnitExists("target") or not UnitIsFriend("player","target") then return end
	
	if UnitIsUnit("player","target") then
		local unitguid = UnitGUID("player")
		roleList[unitguid] = {role, SETTER_MANUAL}
		mode = role
		self:DebugPrint(mode.." mode active.")
		return
	end
	
	if UnitInRaid("target") then
		for i=1, GetNumGroupMembers() do
			if UnitIsUnit("target",raidid[i]) then
				local unitguid = UnitGUID(raidid[i])
				roleList[unitguid] = {role, SETTER_MANUAL}
				self:DebugPrint((UnitName(raidid[i]))..","..role.." role set.")
				if UnitIsUnit("player",raidid[i]) then
					mode = role
					self:DebugPrint(mode.." mode active.")
				end
				return
			end
		end
	elseif UnitInParty("target") then
		for i=1, GetNumSubgroupMembers() do
			if UnitIsUnit("target",partyid[i]) then
				local unitguid = UnitGUID(partyid[i])
				roleList[unitguid] = {role, SETTER_MANUAL}
				self:DebugPrint((UnitName(partyid[i]))..","..role.." role set.")
				return
			end
		end
	end
end

Fragile3.SlashOptions = {
	type = "group",
	name = L["Slash Command"],
	order = -3,
	args = {
		enable = {
			type = "execute",
			order = 1,
			name = L["Enable Fragile3"],
			desc = L["Enable Fragile3"].." ( /fragile enable )",
			func = function() 
				Fragile3:Enable() 
				db.Enabled = true
			end, 
			guiHidden = true,
		},
		disable = {
			type = "execute",
			order = 2,
			name = L["Disable Fragile3"],
			desc = L["Disable Fragile3"].." ( /fragile disable )",
			func = function() 
				Fragile3:Disable() 
				db.Enabled = false
			end,
			guiHidden = true,
		},
		config = {
			type = "execute",
			order = 3,
			name = L["Configure"],
			desc = L["Open the configuration dialog"].." ( /fragile config )",
			func = function() InterfaceOptionsFrame_OpenToCategory(Fragile3.optionsFrames.Fragile) end,
			guiHidden = true,
		},
		mode = {
			type = "execute",
			order = 4,
			name = L["Set mode"],
			desc = L["Show mode selection (in group only)"].." ( /fragile mode )",
			func = function() 
				if groupStatus == SOLO then return end
				if not Fragile3ModePopup:IsShown() then
					Fragile3ModePopup:Show()
				end
			end,
			guiHidden = true,
		},
		dumproles = {
			type = "execute",
			order = 5,
			name = L["Dump Roles"],
			desc = L["Prints the current group roles (debug)"].." ( /fragile dumproles )",
			func = function() Fragile3:DumpRoles() end,
			guiHidden = true,
		},
	}
}

function Fragile3.CreateOptions()
	if not Fragile3.Options then
		Fragile3.Options = {
			type = "group",
			name = "Fragile3 v"..Fragile3.version,
			args = {
				General = {
					order = 1,
					type = "group",
					name = L["General Settings"],
					desc = L["General Settings"],
					args = {
						Enabled = {
							order = 1,
							type = "toggle",
							name = L["Enable"],
							desc = L["Enable/Disable Fragile3"],
							get = function(info) return db.Enabled end,
							set = function(info, value)
								db.Enabled = value
								if db.Enabled then
									if not Fragile3:IsEnabled() then
										Fragile3:Enable()
									end
								else
									if Fragile3:IsEnabled() then
										Fragile3:Disable()
									end
								end
							end,
						},
						UseInRaid = {
							order = 2,
							name = L["Use In Raids"],
							desc = L["Enable Alerts in Raid."],
							type = "toggle",
							get = function(info) return db.UseInRaid end,
							set = function(info, value)
								db.UseInRaid = value
							end,
						},
						Sensitivity = {
							order = 3,
							name = L["Sensitivity"],
							desc = L["Controls sensitivity (Low -> High): \n1.On Aggro, 2.On Threat, \n3.On Targetting."],
							type = "range",
							min = 1, max = 3, step = 1,
							get = function(info) return db.Sensitivity end,
							set = function(info, value)
								db.Sensitivity = value
								if db.Sensitivity == 1 then
									warnstatus = 1 -- warn on aggro
								elseif db.Sensitivity == 2 then
									warnstatus = 0 -- warn on passing tank threat
								elseif db.Sensitivity == 3 then
									warnstatus = -1 -- warn on targeting (LibBanzai)
								end
								Fragile3:GROUP_ROSTER_UPDATE()						
							end,
						},
						Alerts = {
							type = "group",
							order = 4,
							name = L["Alert Settings"],
							desc = L["Alert Settings"],
							args = {
								Sound = {
									order = 1,
									name = L["Sound"],
									desc = L["Enable Sound Alerts."],
									type = "toggle",
									get = function(info) return db.Alerts.Sound end,
									set = function(info, value)
										db.Alerts.Sound = value
									end,
								},
								Throttle = {
									order = 2,
									name = L["Throttle"],
									desc = L["How soon Alerts can repeat (seconds)."],
									type = "range",
									min = 0.5, max = 5, step = 0.5,
									get = function(info) return db.Alerts.Throttle end,
									set = function(info, value)
										db.Alerts.Throttle = value
									end,
								},
								RoleVoices = {
									order = 3,
									name = L["Role Voices"],
									desc = L["Use Role-specific Voices."],
									type = "toggle",
									get = function(info) return db.Alerts.RoleVoices end,
									set = function(info, value)
										db.Alerts.RoleVoices = value
									end,
								},
								GenderVoices = {
									order = 4,
									name = L["Gender Voices"],
									desc = L["Use Gender-specific Voices."],
									type = "toggle",
									get = function(info) return db.Alerts.GenderVoices end,
									set = function(info, value)
										db.Alerts.GenderVoices = value
									end,
								},
								CClassColors = {
									order = 6,
									name = L["!ClassColors"],
									desc = L["Use the !ClassColors addon for coloring."],
									type = "toggle",
									get = function(info) return db.Alerts.CClassColors end,
									set = function(info, value)
										if CUSTOM_CLASS_COLORS then
											db.Alerts.CClassColors = value
											if db.Alerts.CClassColors then	
												RAID_CLASS_COLORS = CUSTOM_CLASS_COLORS
											else
												RAID_CLASS_COLORS = _G.RAID_CLASS_COLORS
											end
										else
											db.Alerts.CClassColors = false	
										end
									end,
								},
								PartyFlash = {
									order = 7,
									name = L["Flash Party Frames"],
									desc = L["Flash Party Frames (Blizzard)."],
									type = "toggle",
									get = function(info) return db.Alerts.PartyFlash end,
									set = function(info, value)
										db.Alerts.PartyFlash = value
									end,
								},
								Message = {
									order = 8,
									name = L["Message"],
									desc = L["Output a text message."],
									type = "toggle",
									get = function(info) return db.Alerts.Message end,
									set = function(info, value)
										db.Alerts.Message = value
									end,
								},
								Test = {
									type = "execute",
									order = 9,
									name = L["Test Options"],
									desc = L["Preview Alerts"],
									func = function() 
										if db.Alerts.Sound then
											local tmpSounds = {"FL3: "..L["(Aggro) Unknown Female"],"FL3: "..L["(Aggro) Damage Male"],"FL3: "..L["(Aggro) Self Female"]}
											PlaySoundFile(LSM:Fetch("sound",tmpSounds[math.random(#tmpSounds)]))
										end
										if db.Alerts.Message then
											local tmpNames = {"|cffFFFFFFHealer|r","|cff9900CCDPSer|r","|cffFFFF33You|r"}
											Fragile3:Pour(L["Aggro on >>"]..tmpNames[math.random(#tmpNames)].."<<",1, 0, 0, nil, 20, "OUTLINE", true)
										end
									end,
								},
								Sounds = {
									order = 5,
									type = "group",
									name = L["Sounds"],
									desc = L["Sound selections"],
									args = {
										aggroUnknownMale = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 1,
											name = MALE..", "..UNKNOWN.." "..ROLE,
											desc = MALE..", "..UNKNOWN.." "..ROLE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroUnknownMale end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroUnknownMale = value
											end,
										},
										aggroUnknownFem = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 2,
											name = FEMALE..", "..UNKNOWN.." "..ROLE,
											desc = FEMALE..", "..UNKNOWN.." "..ROLE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroUnknownFem end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroUnknownFem = value
											end,
										},
										aggroHealerMale = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 3,
											name = HEALER..", "..MALE,
											desc = HEALER..", "..MALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroHealerMale end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroHealerMale = value
											end,
										},
										aggroHealerFem = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 4,
											name = HEALER..", "..FEMALE,
											desc = HEALER..", "..FEMALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroHealerFem end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroHealerFem = value
											end,
										},
										aggroDPSMale = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 5,
											name = DAMAGER..", "..MALE,
											desc = DAMAGER..", "..MALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroDPSMale end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroDPSMale = value
											end,
										},
										aggroDPSFem = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 6,
											name = DAMAGER..", "..FEMALE,
											desc = DAMAGER..", "..FEMALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroDPSFem end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroDPSFem = value
											end,
										},
										aggroOverMale = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 7,
											name = L["Overaggro"]..", "..MALE,
											desc = L["Overaggro"]..", "..MALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroOverMale end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroOverMale = value
											end,
										},
										aggroOverFem = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 8,
											name = L["Overaggro"]..", "..FEMALE,
											desc = L["Overaggro"]..", "..FEMALE,
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroOverFem end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroOverFem = value
											end,
										},
										aggroUnknown = {
											type = "select", dialogControl = 'LSM30_Sound',
											order = 9,
											name = UNKNOWN,
											desc = L["Both role and gender Unknown."],
											values = AceGUIWidgetLSMlists.sound,
											get = function(info) return db.Alerts.Sounds.aggroUnknown end,
											set = function(info, value) 
												db.Alerts.Sounds.aggroUnknown = value
											end,
										},
									},
								},
								Output = Fragile3:GetSinkAce3OptionsDataTable(),
							},
						},
						TankMode = {
							type = "group",
							order = 5,
							name = TANK.." "..MODE,
							desc = TANK.." "..MODE.." "..SETTINGS,
							args = {
								HealerAlert = {
									order = 1,
									name = HEALER.." "..L["Alert"],
									desc = HEALER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.TankMode.HealerAlert end,
									set = function(info, value)
										db.TankMode.HealerAlert = value
									end,
								},
								DPSAlert = {
									order = 2,
									name = DAMAGER.." "..L["Alert"],
									desc = DAMAGER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.TankMode.DPSAlert end,
									set = function(info, value)
										db.TankMode.DPSAlert = value
									end,
								},
								UnknownAlert = {
									order = 3,
									name = UNKNOWN.." "..L["Alert"],
									desc = UNKNOWN.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.TankMode.UnknownAlert end,
									set = function(info, value)
										db.TankMode.UnknownAlert = value
									end,
								},
							},
						},
						HealerMode = {
							type = "group",
							order = 6,
							name = HEALER.." "..MODE,
							desc = HEALER.." "..MODE.." "..SETTINGS,
							args = {
								SelfAlert = {
									order = 1,
									name = L["Overaggro"].." "..L["Alert"],
									desc = L["Overaggro"].." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.HealerMode.SelfAlert end,
									set = function(info, value)
										db.HealerMode.SelfAlert = value
									end,
								},
								HealerAlert = {
									order = 2,
									name = HEALER.." "..L["Alert"],
									desc = HEALER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.HealerMode.HealerAlert end,
									set = function(info, value)
										db.HealerMode.HealerAlert = value
									end,
								},
								DPSAlert = {
									order = 3,
									name = DAMAGER.." "..L["Alert"],
									desc = DAMAGER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.HealerMode.DPSAlert end,
									set = function(info, value)
										db.HealerMode.DPSAlert = value
									end,
								},
								UnknownAlert = {
									order = 4,
									name = UNKNOWN.." "..L["Alert"],
									desc = UNKNOWN.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.HealerMode.UnknownAlert end,
									set = function(info, value)
										db.HealerMode.UnknownAlert = value
									end,
								},
							},
						},
						DPSMode = {
							type = "group",
							order = 7,
							name = DAMAGER.." "..MODE,
							desc = DAMAGER.." "..MODE.." "..SETTINGS,
							args = {
								SelfAlert = {
									order = 1,
									name = L["Overaggro"].." "..L["Alert"],
									desc = L["Overaggro"].." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.DPSMode.SelfAlert end,
									set = function(info, value)
										db.DPSMode.SelfAlert = value
									end,
								},
								HealerAlert = {
									order = 2,
									name = HEALER.." "..L["Alert"],
									desc = HEALER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.DPSMode.HealerAlert end,
									set = function(info, value)
										db.DPSMode.HealerAlert = value
									end,
								},
								DPSAlert = {
									order = 3,
									name = DAMAGER.." "..L["Alert"],
									desc = DAMAGER.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.DPSMode.DPSAlert end,
									set = function(info, value)
										db.DPSMode.DPSAlert = value
									end,
								},
								UnknownAlert = {
									order = 4,
									name = UNKNOWN.." "..L["Alert"],
									desc = UNKNOWN.." "..L["Alert"],
									type = "toggle",
									get = function(info) return db.DPSMode.UnknownAlert end,
									set = function(info, value)
										db.DPSMode.UnknownAlert = value
									end,
								},
							},
						},
					},
				},
			},
		}
		Fragile3.Options.args.General.args.Alerts.args.Output.order = 10
		Fragile3.Options.args.General.args.Alerts.args.Output.inline = true
		Fragile3.Options.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Fragile3.db)
	end
	return Fragile3.Options
end

function Fragile3:SetupOptions()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Fragile3", self.CreateOptions)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Fragile3SlashCommand", self.SlashOptions, "fragile", "fl3")

	-- order determines the order in Blizzard Options
	local ACD3 = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.Fragile = ACD3:AddToBlizOptions("Fragile3", "Fragile3", nil, "General")
	self.optionsFrames.Profiles = ACD3:AddToBlizOptions("Fragile3", L["Profiles"], "Fragile3", "Profiles")
-- 	self.optionsFrames.Help = ACD3:AddToBlizOptions("Fragile3", L["Help File"], self.version, "Help")

	self.SetupOptions = nil

end

-- Frames functions
function Fragile3.EnableModeButton(button)
	button.permDisabled = false;
	button:Enable();
	SetDesaturation(button:GetNormalTexture(), false);
	button.cover:Hide();
	button.checkButton:Show();
	button.checkButton:Enable();
	if ( button.background ) then
		button.background:Show();
	end
end
function Fragile3.PermaDisableModeButton(button)
	button.permDisabled = true;
	button:Disable();
	SetDesaturation(button:GetNormalTexture(), true);
	button.cover:Show();
	button.cover:SetAlpha(0.7);
	button.checkButton:Hide();
	button.checkButton:Disable();
	if ( button.background ) then
		button.background:Hide();
	end
end

function Fragile3.ModeCheckButton_OnClick(button, mbutton)
	-- check this button, uncheck others
	local check = button:GetChecked()	
	Fragile3ModePopupModeButtonTank.checkButton:SetChecked(nil)
 	Fragile3ModePopupModeButtonHealer.checkButton:SetChecked(nil)
 	Fragile3ModePopupModeButtonDPS.checkButton:SetChecked(nil)
 	button:SetChecked(check)
end

function Fragile3.ModeCheckPopupAccept_OnClick()
	-- set the mode according to checkbox
	local unitguid = UnitGUID("player")
	if Fragile3ModePopupModeButtonTank.checkButton:GetChecked() then
		mode = TANK
	elseif Fragile3ModePopupModeButtonHealer.checkButton:GetChecked() then
		mode = HEALER
	elseif Fragile3ModePopupModeButtonDPS.checkButton:GetChecked() then
		mode = DAMAGER
	else
		mode = UNKNOWN
	end
	roleList[unitguid] = (mode == UNKNOWN) and {mode, SETTER_DEFAULT} or {mode, SETTER_MANUAL}
	Fragile3ModePopup:Hide();
end

function Fragile3.ModeCheckPopupDecline_OnClick()
	Fragile3ModePopup:Hide();
end

function Fragile3.ModeCheckPopup_Update()
	-- set permadisable flag based on class.
	Fragile3ModePopupHeader:SetText(L["Mode Selection (Pick your Role)"]);
	local canBeTank, canBeHealer, canBeDPS 
	canBeTank, canBeHealer, canBeDPS = UnitGetAvailableRoles("player");

	if ( canBeTank ) then
		Fragile3.EnableModeButton(Fragile3ModePopupModeButtonTank);
	else
		Fragile3.PermaDisableModeButton(Fragile3ModePopupModeButtonTank);
	end
	
	if ( canBeHealer ) then
		Fragile3.EnableModeButton(Fragile3ModePopupModeButtonHealer);
	else
		Fragile3.PermaDisableModeButton(Fragile3ModePopupModeButtonHealer);
	end
	
	if ( canBeDPS ) then
		Fragile3.EnableModeButton(Fragile3ModePopupModeButtonDPS);
	else
		Fragile3.PermaDisableModeButton(Fragile3ModePopupModeButtonDPS);
	end
	if mode and mode ~= UNKNOWN then
		if mode == TANK and not	Fragile3ModePopupModeButtonTank.checkButton:GetChecked() then
			Fragile3ModePopupModeButtonTank.checkButton:SetChecked(1)
			Fragile3.ModeCheckButton_OnClick(Fragile3ModePopupModeButtonTank.checkButton)
		elseif mode == HEALER and not Fragile3ModePopupModeButtonHealer.checkButton:GetChecked() then
			Fragile3ModePopupModeButtonHealer.checkButton:SetChecked(1)
			Fragile3.ModeCheckButton_OnClick(Fragile3ModePopupModeButtonHealer.checkButton)
		elseif mode == DAMAGER and not Fragile3ModePopupModeButtonDPS.checkButton:GetChecked() then
			Fragile3ModePopupModeButtonDPS.checkButton:SetChecked(1)
			Fragile3.ModeCheckButton_OnClick(Fragile3ModePopupModeButtonDPS.checkButton)
		end
	end	
end
